"""
Author: Xsarz

Enjoy using!
"""

from .utils import exceptions, objects, requester
from .client import Client
from .async_client import AsyncClient


from os import system as s
from json import loads
from requests import get
__title__ = 'stationhead'
__author__ = 'Xsarz'
__license__ = 'MIT'
__copyright__ = 'Copyright 2023 Xsarz'
__version__ = '1.1'


__newest__ = loads(get("https://pypi.org/pypi/stationhead/json").text)["info"]["version"]
if __version__ != __newest__:
	s('cls || clear')
	print(f'\033[38;5;214m{__title__} made by {__author__}\nPlease update the library. Your version: {__version__}  A new version:{__newest__}\033[0m')


from .exceptions import *

from requests import Session
from aiohttp import ClientSession
from json import dumps

class Requester:
	def __init__(self):
		self.api = "https://production1.stationhead.com"
		self.sid = None
		self.session = Session()
		self.asyncSession = ClientSession()

	def headers(self):
		headers = {
			"app-platform": "android",
			"app-version": "1.2.48",
			"content-type": "application/json"
		}
		if self.sid:headers["authorization"] = self.sid

		return headers


	def make_request(self, method: str, endpoint: str, body: dict = None):

		response = self.session.request(method, f"{self.api}{endpoint}", data=dumps(body), headers=self.headers())
		return checkException(response.text) if response.status_code != 200 else response


	async def make_async_request(self, method: str, endpoint: str, body: dict = None):
		response = await self.asyncSession.request(method, f"{self.api}{endpoint}", data=dumps(body), headers=self.headers())
		return checkException(await response.text()) if response.status != 200 else response


from .utils.requester import Requester
from .utils import objects, exceptions


from json import loads

class Client:
	def __init__(self):
		self.req = Requester()


	def login(self, email: str, password: str):

		data = {
			"clear_broadcast_data": True,
			"resalt": True,
			"password": password,
			"username": email
		}
		resp = objects.profile(loads(self.req.make_request(method="POST", endpoint="/login", body=data).text))
		self.req.sid = resp.sid
		return resp


	def register(self, email: str, password: str, nickname: str):

		data = {
			"email": email,
			"password": password,
			"handle": nickname
		}

		resp = objects.profile(loads(self.req.make_request(method="POST", endpoint="/user", body=data).text))
		return resp


from json import loads

class UnknownError(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class InvalidEmailOrPassword(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)



errors = {
	"2050": InvalidEmailOrPassword,
}

def checkException(data):
	try:
		data = loads(data)
		code = data["error"]["code"]
	except:
		raise UnknownError(data)
	if code in errors: raise errors[code](data)
	else:raise UnknownError(data)


from .utils.requester import Requester
from .utils import objects, exceptions
from asyncio import get_event_loop, new_event_loop

from json import loads

class AsyncClient:
	def __init__(self):
		self.req = Requester()


	def __del__(self):
		try:
			loop = get_event_loop()
			loop.run_until_complete(self.close_session())
		except RuntimeError:
			loop = new_event_loop()
			loop.run_until_complete(self.close_session())

	async def close_session(self):
		if not self.req.asyncSession.closed: await self.req.asyncSession.close()


	async def login(self, email: str, password: str):

		data = {
			"clear_broadcast_data": True,
			"resalt": True,
			"password": password,
			"username": email
		}
		resp = await self.req.make_async_request(method="POST", endpoint="/login", body=data)
		resp = objects.profile(loads(await resp.text()))
		self.req.sid = resp.sid
		return resp


	async def register(self, email: str, password: str, nickname: str):

		data = {
			"email": email,
			"password": password,
			"handle": nickname
		}
		resp = await self.req.make_async_request(method="POST", endpoint="/user", body=data)
		return objects.profile(loads(await resp.text()))


from setuptools import setup, find_packages

with open("README.md", "r") as file:
	long_description = file.read()

link = 'https://github.com/xXxCLOTIxXx/stationhead/archive/refs/heads/main.zip'
ver = '1.1'

setup(
	name = "stationhead",
	version = ver,
	url = "https://github.com/xXxCLOTIxXx/stationhead",
	download_url = link,
	license = "MIT",
	author = "Xsarz",
	author_email = "xsarzy@gmail.com",
	description = "Library for creating stationhead bots and scripts.",
	long_description = long_description,
	long_description_content_type = "text/markdown",
	keywords = [
		"stationhead.py",
		"stationhead",
		"stationhead-py",
		"stationhead-bot",
		"api",
		"python",
		"python3",
		"python3.x",
		"xsarz",
		"official",
		"sync"
	],
	install_requires = [
		"requests"
	],
	packages = find_packages()
)





class profile:
	def __init__(self, data: dict):
		self.json = data
		self.sid = self.json.get("", None)
		self.tipping_salt = self.json.get("tipping_salt", None)
		self.id = self.json.get("id", None)
		self.email = self.json.get("email", None)
		self.contact_email = self.json.get("contact_email", None)
		self.is_premium = self.json.get("is_premium", None)
		self.country = self.json.get("country", None)
		self.type = self.json.get("type", None)
		self.response_timestamp = self.json.get("response_timestamp", None)
		self.suspension = self.json.get("suspension", None)
		self.timezone = self.json.get("timezone", None)
		self.spotify_client_id = self.json.get("spotify_client_id", None)
		self.app_data = self.json.get("app_data", {})